import XMonad hiding ((|||))
import qualified XMonad.StackSet as W
import qualified Data.Map        as M
import XMonad.Config.Desktop
import XMonad.Layout.Gaps
import XMonad.Hooks.EwmhDesktops
import XMonad.Hooks.DynamicLog
import XMonad.Hooks.ManageDocks
import XMonad.Util.Run(spawnPipe)
import XMonad.Util.EZConfig(additionalKeys, additionalKeysP, additionalMouseBindings)
import System.IO
import System.Exit
import XMonad.Actions.GroupNavigation
import XMonad.Layout.Tabbed
import XMonad.Hooks.InsertPosition
import XMonad.Layout.SimpleDecoration (shrinkText)
import XMonad.Util.WorkspaceCompare
import XMonad.Hooks.ManageHelpers
import XMonad.Actions.PhysicalScreens
import Data.Default
import XMonad.Layout.LayoutCombinators
import XMonad.Layout.NoBorders
import XMonad.Layout.Spacing
import XMonad.Layout.ResizableTile
import XMonad.Layout.TwoPane
import XMonad.Layout.BinarySpacePartition
import XMonad.Layout.Dwindle
import XMonad.Actions.SinkAll
import XMonad.Util.NamedScratchpad
import XMonad.StackSet as W 

myTabConfig1 = def { activeColor = "#268BD2"
                  , inactiveColor = "#268BD2"
                  , urgentColor = "#FDF6E3"
                  , activeBorderColor = "##268BD2"
                  , inactiveBorderColor = "#4#268BD2"
                  , urgentBorderColor = "#268BD2"
                  , activeTextColor = "#2F3D44"
                  , inactiveTextColor = "#2F3D44"
                  , urgentTextColor = "#2F3D44"
                  , fontName = "xft:Noto Sans CJK:size=10:antialias=true"
                  }
myTabConfig = def { activeColor = "#268BD2"
                  , inactiveColor = "#2F3D44"
                  , urgentColor = "#FDF6E3"
                  , activeBorderColor = "#454948"
                  , inactiveBorderColor = "#454948"
                  , urgentBorderColor = "#268BD2"
                  , activeTextColor = "#80FFF9"
                  , inactiveTextColor = "#1ABC9C"
                  , urgentTextColor = "#1ABC9C"
                  , fontName = "xft:Noto Sans CJK:size=10:antialias=true"
                  }

----ez
scratchpads :: [NamedScratchpad]
scratchpads = [
 NS "pavucontrol" "pavucontrol" (className =? "Pavucontrol")
        (customFloating $ W.RationalRect (1/4) (1/4) (2/4) (2/4))
     ]       
----ig


myLayout = avoidStruts $
           gaps [(U,30), (R,3), (L,3)] $
  noBorders (tabbed shrinkText myTabConfig)
  ||| noBorders (tabbedBottom shrinkText myTabConfig1)
  ||| tiled
  ||| Mirror tiled
  ||| noBorders Full 
  ||| twopane
  ||| Mirror twopane
  ||| emptyBSP
  ||| Spiral L XMonad.Layout.Dwindle.CW (3/2) (11/10) -- L means the non-main windows are put to the left.

  where
     -- The last parameter is fraction to multiply the slave window heights
     -- with. Useless here.
     tiled = spacing 3 $ ResizableTall nmaster delta ratio []
     -- In this layout the second pane will only show the focused window.
     twopane = spacing 3 $ TwoPane delta ratio
     -- The default number of windows in the master pane
     nmaster = 1
     -- Default proportion of screen occupied by master pane
     ratio   = 1/2
     -- Percent of screen to increment by when resizing panes
     delta   = 3/100

myPP = def { ppCurrent = xmobarColor "#1ABC9C" "" . wrap "[" "]"
           , ppTitle = xmobarColor "#1ABC9C" "" . shorten 60
           , ppVisible = wrap "(" ")"
           , ppUrgent  = xmobarColor "red" "yellow"
           , ppSort = getSortByXineramaPhysicalRule def
           }
myDFloat = doRectFloat (W.RationalRect (1/6) (1/6) (2/3) (2/3))
myWorkspaces =  ["1","2","3","4","5","6","7","8","9"]
myManageHook = composeAll . concat $
  [ [isDialog --> myDFloat]
  , [className =? c --> doCenterFloat | c <- myCFloats]
  , [title =? t --> doCenterFloat | t <- myTFloats]
  , [resource =? r --> doCenterFloat | r <- myRFloats]
  , [(className =? i <||> title =? i <||> resource =? i) --> doIgnore | i <- myIgnores]
  , [(className =? x <||> title =? x <||> resource =? x) --> doShift (myWorkspaces !! 0) | x <- my1Shifts]
  , [(className =? x <||> title =? x <||> resource =? x) --> doShift (myWorkspaces !! 1) | x <- my2Shifts]
  , [(className =? x <||> title =? x <||> resource =? x) --> doShift (myWorkspaces !! 2) | x <- my3Shifts]
  , [(className =? x <||> title =? x <||> resource =? x) --> doShift (myWorkspaces !! 3) | x <- my4Shifts]
  , [(className =? x <||> title =? x <||> resource =? x) --> doShift (myWorkspaces !! 4) | x <- my5Shifts]
  , [(className =? x <||> title =? x <||> resource =? x) --> doShift (myWorkspaces !! 5) | x <- my6Shifts]
  , [(className =? x <||> title =? x <||> resource =? x) --> doShift (myWorkspaces !! 6) | x <- my7Shifts]
  , [(className =? x <||> title =? x <||> resource =? x) --> doShift (myWorkspaces !! 7) | x <- my8Shifts]
  , [(className =? x <||> title =? x <||> resource =? x) --> doShift (myWorkspaces !! 8) | x <- my9Shifts]
  ]
  where
--classes / titles / resources
  myCFloats = []
  myTFloats = []
  myRFloats = []
  myIgnores = []
  my1Shifts = ["Firefox","Chromium"]
  my2Shifts = [""]
  my3Shifts = ["mail","mumble"]
  my4Shifts = ["Gimp","MPlayer","code-oss"]
  my5Shifts = ["remmina","xfreerdp","rdesktop"]
  my6Shifts = ["VirtualBox Manager","VirtualBox","virt-manager"]
  my7Shifts = []
  my8Shifts = []
  my9Shifts = []
  

myKeys conf@(XConfig {XMonad.modMask = modm}) = M.fromList $

    -- launch a terminal
    [ --((modm .|. shiftMask, xK_F4), spawn "alacritty")

    -- close focused window
    ((modm, xK_q     ), kill)

     -- Rotate through the available layout algorithms
    , ((modm,               xK_space ), sendMessage NextLayout)

    --  Reset the layouts on the current workspace to default
    , ((modm .|. shiftMask, xK_space ), setLayout $ XMonad.layoutHook conf)
    , ((modm .|. shiftMask, xK_h), sendMessage $ JumpToLayout "Mirror Tall")
    , ((modm .|. shiftMask, xK_v), sendMessage $ JumpToLayout "Tall")
    , ((modm .|. shiftMask, xK_f), sendMessage $ JumpToLayout "Full")
    , ((modm .|. shiftMask, xK_t), sendMessage $ JumpToLayout "Tabbed Simplest")
    , ((modm,        xK_u), sinkAll)
    -- Resize viewed windows to the correct size
    , ((modm,            xK_n), refresh)

    -- Move focus to the next window
    , ((modm,               xK_Tab   ), windows W.focusDown)

    -- Move focus to the next window
    --, ((modm,               xK_j     ), windows W.focusDown)

    -- Move focus to the previous window
    --, ((modm,               xK_k     ), windows W.focusUp  )

    -- Move focus to the master window
    --, ((modm,               xK_m     ), windows W.focusMaster  )

    -- Swap the focused window and the master window
    --, ((modm,               xK_Return), windows W.swapMaster)

    -- Swap the focused window with the next window
    , ((modm .|. shiftMask, xK_j     ), windows W.swapDown  )

    -- Swap the focused window with the previous window
    , ((modm .|. shiftMask, xK_k     ), windows W.swapUp    )

    -- Shrink the master area
    , ((modm,               xK_Up     ), sendMessage Shrink)

    -- Expand the master area
    , ((modm,               xK_Down     ), sendMessage Expand)

    -- Shrink and expand ratio between the secondary panes, for the ResizableTall layout
    , ((modm,        xK_Right ), sendMessage MirrorShrink)
    , ((modm,        xK_Left), sendMessage MirrorExpand)

    -- Push window back into tiling
    --, ((modm,               xK_t     ), withFocused $ windows . W.sink)

    -- Increment the number of windows in the master area
    --, ((modm              , xK_comma ), sendMessage (IncMasterN 1))

    -- Deincrement the number of windows in the master area
    --, ((modm              , xK_period), sendMessage (IncMasterN (-1)))

    -- Toggle the status bar gap
    -- Use this binding with avoidStruts from Hooks.ManageDocks.
    -- See also the statusBar function from Hooks.DynamicLog.
    --, ((modm              , xK_b     ), sendMessage ToggleStruts)
    --, ((controlMask, xK_Print), spawn "sleep 0.2; scrot -s")
    --, ((0, xK_Print), spawn "scrot") -- 0 means no extra modifier key needs to be pressed in this case.
    --, ((modm, xK_F3), spawn "pcmanfm")
    --, ((modm.|. shiftMask, xK_F3), spawn "gksu pcmanfm")
    ]

    ++
      [((m .|. modm, k), windows $ f i)
      | (i, k) <- zip (XMonad.workspaces conf) [xK_1 .. xK_9]
      , (f, m) <- [(W.greedyView, 0), (W.shift, shiftMask)]]
    ++
      [((m .|. modm, key), f sc)
      | (key, sc) <- zip [xK_a, xK_s, xK_d] [0..]
      -- Order screen by physical order instead of arbitrary numberings.
      , (f, m) <- [(viewScreen def, 0), (sendToScreen def, shiftMask)]]

main = do
    xmproc <- spawnPipe "xmobar"
    xmonad $ ewmh def
        { modMask = mod4Mask
        , keys = myKeys
        --, manageHook = (manageDocks <+> myManageHook <+> namedScratchpad ManageHook scratchpads)
        , manageHook = (myManageHook <+> manageDocks ) <+> namedScratchpadManageHook scratchpads
        , layoutHook = myLayout
        , focusFollowsMouse  = False
        , clickJustFocuses   = True
        , handleEventHook = handleEventHook def <+> docksEventHook
        , logHook = dynamicLogWithPP myPP {
                                          ppOutput = hPutStrLn xmproc
                                          }
                        >> historyHook
        , terminal = "lxterminal"
       , startupHook = do
            startupHook desktopConfig
            spawn "sxhkd -c /home/laci/.config/sxhkd/sxhkdrc.xmonad"
            --adjustEventInput
        -- This is the color of the borders of the windows themselves.
        , normalBorderColor  = "#2f3d44"
        , focusedBorderColor = "#1ABC9C"
        }
        `additionalKeysP`
        [
          ("M1-<Space>", spawn "rofi -combi-modi window,run,drun -show combi -modi combi")
          , ("C-M1-<Space>", spawn "rofi -show run")
          -- Restart xmonad. This is the same keybinding as from i3
          --, ("M-S-e", spawn "xmonad --recompile; xmonad --restart")
          , ("M-q", kill)
          , ("M-a", spawn "audacity")
          --, ("M-'", windows W.swapMaster)
          --, ("M1-<Tab>", nextMatch History (return True))
          --, ("M-<Return>", spawn "lxterminal")
          -- Make it really hard to mispress...
          , ("M-v", namedScratchpadAction scratchpads "pavucontrol")
          , ("M-S-q", io (exitWith ExitSuccess))
         ] `additionalMouseBindings`
        [ ((mod4Mask, button4), (\w -> windows W.focusUp))
        , ((mod4Mask, button5), (\w -> windows W.focusDown))
        ]



          
